/* Osm4j is a library that provides utilities to handle Openstreetmap data in java.
 *
 * Copyright (C) 2011  Sebastian Kuerten
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package model.address.android;

import java.util.ArrayList;
import java.util.List;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

/**
 * @author Sebastian Kuerten (sebastian.kuerten@fu-berlin.de)
 * 
 */
public class SqPlace {

	private int id;
	private String name;
	private int type;
	private SqCity city;
	private SqPostcode postcode;

	/**
	 * @return the id
	 */
	public int getId() {
		return this.id;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * @return the type of this place (0 = city, 1 = town, 2 = village)
	 */
	public int getType() {
		return type;
	}

	/**
	 * @return the city this place is in.
	 */
	public SqCity getCity() {
		return city;
	}

	/**
	 * @return the postal code this place is in.
	 */
	public SqPostcode getPostcode() {
		return postcode;
	}

	private static final String select =
			"SELECT places.id, places.name, places.type, " +
					"cities.id, cities.name, postcodes.id, postcodes.code" +
					" FROM places JOIN cities ON places.city_id = cities.id" +
					" LEFT OUTER JOIN postcodes ON places.postcode_id = postcodes.id";

	/**
	 * Fetch a place by its id.
	 * 
	 * @param db
	 *            a Connection to use
	 * @param id
	 *            the id of the place
	 * 
	 * @return the place with the denoted id or null
	 */
	public static SqPlace getPlace(SQLiteDatabase db, int id) {
		return getPlace(db, " WHERE places.id = " + id);
	}

	private static SqPlace getPlace(SQLiteDatabase db, String whereclause) {
		String stmt = select + whereclause;
		Cursor cursor = db.rawQuery(stmt, null);
		if (!cursor.moveToNext()) {
			cursor.close();
			return null;
		}
		SqPlace place = getPlace(cursor);
		cursor.close();
		return place;
	}

	private static List<SqPlace> getPlaces(SQLiteDatabase db, String whereclause) {
		String stmt = select + whereclause;
		Cursor cursor = db.rawQuery(stmt, null);

		List<SqPlace> list = new ArrayList<SqPlace>();
		while (cursor.moveToNext()) {
			list.add(getPlace(cursor));
		}
		cursor.close();

		return list;
	}

	private static SqPlace getPlace(Cursor cursor) {
		SqPlace place = new SqPlace();

		place.id = cursor.getInt(0);
		place.name = cursor.getString(1);
		place.type = cursor.getInt(2);

		place.city = new SqCity(
					cursor.getInt(3), cursor.getString(4));

		place.postcode = cursor.isNull(6) ? null
				: new SqPostcode(cursor.getInt(5), cursor.getString(6));

		return place;
	}

	/**
	 * Fetch a list of resulting cities.
	 * 
	 * @param db
	 *            a Connection to use
	 * @param querystring
	 *            the name to search for.
	 * @param exact
	 *            whether to use wildcards in like-search
	 * @param limit
	 *            a number limiting the number of returned values.
	 * @return the list of results.
	 */
	public static List<SqPlace> getPlaces(SQLiteDatabase db, String querystring, boolean exact,
			int limit) {
		String query = exact ? querystring : "%" + querystring + "%";
		String whereclause = " WHERE places.name LIKE '" + query + "' LIMIT " + limit;

		List<SqPlace> places = getPlaces(db, whereclause);

		return places;
	}

	@Override
	public int hashCode() {
		return this.id;
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof SqPlace) {
			SqPlace o = (SqPlace) other;
			return o.id == this.id;
		}
		return false;
	}
}
