/* Osm4j is a library that provides utilities to handle Openstreetmap data in java.
 *
 * Copyright (C) 2011  Sebastian Kuerten
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package model.address.android;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

/**
 * @author Sebastian Kuerten (sebastian.kuerten@fu-berlin.de)
 * 
 */
public class SqRoad {

	private int id;
	private String name;
	private double lat, lon;
	private int cityId;

	/**
	 * @return the id
	 */
	public int getId() {
		return this.id;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * @return the latitude.
	 */
	public double getLat() {
		return this.lat;
	}

	/**
	 * @return the longitude.
	 */
	public double getLon() {
		return this.lon;
	}

	/**
	 * @return the city's id.
	 */
	public int getCityId() {
		return this.cityId;
	}

	private Set<SqBorough> boroughs = null;
	private Set<SqPostcode> postcodes = null;

	/**
	 * Retrieve the set of boroughs for this road.
	 * 
	 * @param db
	 *            a connection to use to query.
	 * @return a set of boroughs.
	 */
	public Set<SqBorough> getBoroughs(SQLiteDatabase db) {
		if (this.boroughs != null) {
			return this.boroughs;
		}
		this.boroughs = new HashSet<SqBorough>();
		String stmt = "select boroughs.id, boroughs.level, boroughs.name " + "from roads "
				+ "join roads_boroughs on roads.id=roads_boroughs.roads_id "
				+ "join boroughs on roads_boroughs.boroughs_id=boroughs.id "
				+ "where roads.id=" + this.id + " and roads.city_id = boroughs.city_id";
		Cursor cursor = db.rawQuery(stmt, null);

		while (cursor.moveToNext()) {
			int bid = cursor.getInt(0);
			int level = cursor.getInt(1);
			String bname = cursor.getString(2);
			this.boroughs.add(new SqBorough(bid, level, bname));
		}

		cursor.close();
		return this.boroughs;
	}

	/**
	 * Retrieve the set of postcodes for this road.
	 * 
	 * @param db
	 *            a connection to use to query.
	 * @return a set of postcodes.
	 */
	public Set<SqPostcode> getPostcodes(SQLiteDatabase db) {
		if (this.postcodes != null) {
			return this.postcodes;
		}
		this.postcodes = new HashSet<SqPostcode>();
		String stmt = "select postcodes.id, postcodes.code " +
				"from roads " +
				"join roads_postcodes on roads.id=roads_postcodes.roads_id " +
				"join postcodes on roads_postcodes.postcodes_id=postcodes.id " +
				"where roads.id=" + this.id;
		Cursor cursor = db.rawQuery(stmt, null);

		while (cursor.moveToNext()) {
			int pid = cursor.getInt(0);
			String pname = cursor.getString(1);
			this.postcodes.add(new SqPostcode(pid, pname));
		}

		cursor.close();
		return this.postcodes;
	}

	/**
	 * Return a String in format A, B, C where A, B, C are boroughs of this road.
	 * 
	 * @param database
	 *            a connection to use
	 * @return a pretty String
	 */
	public String getBoroughsAsString(SQLiteDatabase database) {
		StringBuilder strb = new StringBuilder();
		if (getBoroughs(database).size() > 0) {
			List<SqBorough> bs = new ArrayList<SqBorough>();
			for (SqBorough b : getBoroughs(database)) {
				bs.add(b);
			}
			Collections.sort(bs, new Comparator<SqBorough>() {

				@Override
				public int compare(SqBorough o1, SqBorough o2) {
					if (o1.getLevel() != o2.getLevel())
						return o1.getLevel() - o2.getLevel();
					return o1.getName().compareTo(o2.getName());
				}
			});
			for (int i = 0; i < bs.size(); i++) {
				SqBorough dbBorough = bs.get(i);
				strb.append(dbBorough.getName());
				if (i < bs.size() - 1) {
					strb.append(", ");
				}
			}
		}
		return strb.toString();
	}

	/**
	 * Return a String in format A, B, C where A, B, C are postal codes of this road.
	 * 
	 * @param database
	 *            a connection to use
	 * @return a pretty String
	 */
	public String getPostcodesAsString(SQLiteDatabase database) {
		StringBuilder strb = new StringBuilder();
		if (getPostcodes(database).size() > 0) {
			List<SqPostcode> bs = new ArrayList<SqPostcode>();
			for (SqPostcode b : getPostcodes(database)) {
				bs.add(b);
			}
			Collections.sort(bs, new Comparator<SqPostcode>() {

				@Override
				public int compare(SqPostcode o1, SqPostcode o2) {
					return o1.getPostcode().compareTo(o2.getPostcode());
				}
			});
			for (int i = 0; i < bs.size(); i++) {
				SqPostcode dbPostcode = bs.get(i);
				strb.append(dbPostcode.getPostcode());
				if (i < bs.size() - 1) {
					strb.append(", ");
				}
			}
		}
		return strb.toString();
	}

	/**
	 * Fetch a list of resulting cities.
	 * 
	 * @param db
	 *            a Connection to use for the query
	 * @param cityId
	 *            the city to search in.
	 * @param querystring
	 *            the name to search for.
	 * @param limit
	 *            a number limiting the number of results.
	 * @return the list of results.
	 */
	public static List<SqRoad> getRoadsByCity(SQLiteDatabase db, int cityId,
			String querystring, int limit) {
		String stmt = "select roads.id, roads.lat, roads.lon, roads.name" +
				" from cities join roads on cities.id = roads.city_id"
				+ " where cities.id = " + cityId + " and roads.name like '%" + querystring
				+ "%' limit " + limit;
		Cursor cursor = db.rawQuery(stmt, null);
		List<SqRoad> list = listFromCursor(cursor);
		cursor.close();
		return list;
	}

	/**
	 * Fetch a list of resulting cities.
	 * 
	 * @param db
	 *            a Connection to use for the query
	 * @param postcodeId
	 *            the postal code to search in.
	 * @param querystring
	 *            the name to search for.
	 * @param limit
	 *            a number limiting the number of results.
	 * @return the list of results.
	 */
	public static List<SqRoad> getRoadsByPostcode(SQLiteDatabase db, int postcodeId,
			String querystring, int limit) {
		String stmt = "select roads.id, roads.lat, roads.lon, roads.name" +
				" from roads join roads_postcodes on(roads.id=roads_postcodes.roads_id)" +
				" join postcodes on(roads_postcodes.postcodes_id=postcodes.id)" +
				" where postcodes.id = " + postcodeId + " and roads.name like '%" + querystring
				+ "%' limit " + limit;
		Cursor cursor = db.rawQuery(stmt, null);
		List<SqRoad> list = listFromCursor(cursor);
		cursor.close();
		return list;
	}

	private static List<SqRoad> listFromCursor(Cursor cursor) {
		List<SqRoad> list = new ArrayList<SqRoad>();
		while (cursor.moveToNext()) {
			SqRoad road = new SqRoad();
			list.add(road);
			road.id = cursor.getInt(0);
			road.lat = cursor.getDouble(1);
			road.lon = cursor.getDouble(2);
			road.name = cursor.getString(3);
		}
		return list;
	}

	/**
	 * Produce a verbose String for detailed output
	 * 
	 * @param database
	 *            a database
	 * @param coordinates
	 *            whether to show coordinates.
	 * @return a pretty string representation
	 */
	public String toVerboseString(SQLiteDatabase database, boolean coordinates) {
		StringBuilder strb = new StringBuilder();
		if (coordinates) {
			strb.append(toString());
		} else {
			strb.append(name);
		}
		String bs = getBoroughsAsString(database);
		if (bs.length() > 0) {
			strb.append(" (");
			strb.append(bs);
			strb.append(")");
		}
		String ps = getPostcodesAsString(database);
		if (ps.length() > 0) {
			strb.append(" (");
			strb.append(ps);
			strb.append(")");
		}
		return strb.toString();
	}

	@Override
	public int hashCode() {
		return this.id;
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof SqRoad) {
			SqRoad o = (SqRoad) other;
			return o.id == this.id;
		}
		return false;
	}
}
